// SPDX-FileCopyrightText: 2021 Carlos Garnacho <carlosg@gnome.org>
//
// SPDX-License-Identifier: GPL-2.0-or-later
//

import Clutter from 'gi://Clutter';
import Gio from 'gi://Gio';
import St from 'gi://St';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';

import {AppSwitchAction} from 'resource:///org/gnome/shell/ui/windowManager.js';
import {EdgeDragAction} from 'resource:///org/gnome/shell/ui/edgeDragAction.js';

import {Extension} from 'resource:///org/gnome/shell/extensions/extension.js';

export default class GestureInhibitorExtension extends Extension {
    constructor(metadata) {
        super(metadata);

        let actions = global.stage.get_actions();

        actions.forEach(a => {
            if (a instanceof AppSwitchAction)
                this._appSwitch = a;
            else if (a instanceof EdgeDragAction &&
                 a._side === St.Side.BOTTOM)
                this._showOsk = a;
            else if (a instanceof EdgeDragAction &&
                 a._side === St.Side.TOP)
                this._unfullscreen = a;
        });

        this._map = [
            {setting: 'overview', action: Main.overview._swipeTracker},
            {setting: 'app-switch', action: this._appSwitch},
            {setting: 'show-osk', action: this._showOsk},
            {setting: 'unfullscreen', action: this._unfullscreen},
            {setting: 'workspace-switch', action: Main.wm._workspaceAnimation._swipeTracker},
        ];

        this._enabledDesc = Object.getOwnPropertyDescriptor(
            Clutter.ActorMeta.prototype, 'enabled');
    }

    _overrideEnabledSetter(obj, set) {
        if (!(obj instanceof Clutter.ActorMeta))
            return;

        const desc = set
            ? {...this._enabledDesc, set}
            : {...this._enabledDesc};
        Object.defineProperty(obj, 'enabled', desc);
    }

    enable() {
        const settings = this.getSettings();

        this._map.forEach(m => {
            settings.bind(m.setting, m.action, 'enabled',
                Gio.SettingsBindFlags.DEFAULT);

            this._overrideEnabledSetter(m.action, function (value) {
                if (settings.get_boolean(m.setting)) {
                    // eslint-disable-next-line no-invalid-this
                    this.set_enabled(value);
                }
            });
        });
    }

    disable() {
        this._map.forEach(m => {
            Gio.Settings.unbind(m.action, 'enabled');
            this._overrideEnabledSetter(m.action);
            m.action.enabled = true;
        });
    }
}
